''' <summary>
''' Eine Farbtabelle als Basis fr einen ToolStripProfessionalRenderer
''' </summary>
''' <remarks>Hier werden harmonische Basisfarben definiert, die eigentlich zum Zeichnen 
''' von MenElementen vorgesehen sind. Man kann so eine Farbtabelle aber auch zum Rendern
''' anderer selbst gezeichneter Controls verwenden. 
''' Besonders an der hier vorgestellten Farbtabelle ist, dass alle enthaltenen Farbtne
''' in einem beliebigen Winkel innerhalb des Farbkreises gleichzeitig "gedreht" werden
''' knnen und die Distanz der Farbtne untereinander verndert werden kann. 
''' Damit bleiben Farbharmonien erhalten, und in gewissem Ramhmen kann der Farb-
''' Kontrast verndert werden.</remarks>
Public Class ColorWheelProfessionalTable
    Inherits ProfessionalColorTable

#Region " Private Felder / Events "

    ''' <summary>
    ''' Event bei nderung der Tabelle
    ''' </summary>
    ''' <remarks>fhrt zum Neuzeichnen abhngiger Controls</remarks>
    Public Event Changed()

    ' Basis-Farbtabelle
    Private m_BaseTable As ProfessionalColorTable = New ExampleColorTable

    ' Mittelwert fr alle Farbtne der Basis-Farbtabelle
    Private m_MidHue As Decimal

    ' Dehnungsfaktor fr die Farben untereinander
    Private m_StretchFactor As Decimal
    ' Farbton fr die einfache Hintergrundfarbe fr Controls
    Private m_ControlHue As Double

    ' Liste mit allen in der ProfessionalColorTable enthaltenen benannten Farben
    Private m_AllColors As New List(Of AHSLColor)

    ' aktuelle Farben
    Private m_ButtonCheckedGradientBegin As AHSLColor
    Private m_ButtonCheckedGradientEnd As AHSLColor
    Private m_ButtonCheckedGradientMiddle As AHSLColor
    Private m_ButtonCheckedHighlight As AHSLColor
    Private m_ButtonCheckedHighlightBorder As AHSLColor
    Private m_ButtonPressedBorder As AHSLColor
    Private m_ButtonPressedGradientBegin As AHSLColor
    Private m_ButtonPressedGradientEnd As AHSLColor
    Private m_ButtonPressedGradientMiddle As AHSLColor
    Private m_ButtonPressedHighlight As AHSLColor
    Private m_ButtonPressedHighlightBorder As AHSLColor
    Private m_ButtonSelectedBorder As AHSLColor
    Private m_ButtonSelectedGradientBegin As AHSLColor
    Private m_ButtonSelectedGradientEnd As AHSLColor
    Private m_ButtonSelectedGradientMiddle As AHSLColor
    Private m_ButtonSelectedHighlight As AHSLColor
    Private m_ButtonSelectedHighlightBorder As AHSLColor
    Private m_CheckBackground As AHSLColor
    Private m_CheckPressedBackground As AHSLColor
    Private m_CheckSelectedBackground As AHSLColor
    Private m_GripDark As AHSLColor
    Private m_GripLight As AHSLColor
    Private m_ImageMarginGradientBegin As AHSLColor
    Private m_ImageMarginGradientEnd As AHSLColor
    Private m_ImageMarginGradientMiddle As AHSLColor
    Private m_ImageMarginRevealedGradientBegin As AHSLColor
    Private m_ImageMarginRevealedGradientEnd As AHSLColor
    Private m_ImageMarginRevealedGradientMiddle As AHSLColor
    Private m_MenuBorder As AHSLColor
    Private m_MenuItemBorder As AHSLColor
    Private m_MenuItemPressedGradientBegin As AHSLColor
    Private m_MenuItemPressedGradientEnd As AHSLColor
    Private m_MenuItemPressedGradientMiddle As AHSLColor
    Private m_MenuItemSelected As AHSLColor
    Private m_MenuItemSelectedGradientBegin As AHSLColor
    Private m_MenuItemSelectedGradientEnd As AHSLColor
    Private m_MenuStripGradientBegin As AHSLColor
    Private m_MenuStripGradientEnd As AHSLColor
    Private m_OverflowButtonGradientBegin As AHSLColor
    Private m_OverflowButtonGradientEnd As AHSLColor
    Private m_OverflowButtonGradientMiddle As AHSLColor
    Private m_RaftingContainerGradientBegin As AHSLColor
    Private m_RaftingContainerGradientEnd As AHSLColor
    Private m_SeparatorDark As AHSLColor
    Private m_SeparatorLight As AHSLColor
    Private m_StatusStripGradientBegin As AHSLColor
    Private m_StatusStripGradientEnd As AHSLColor
    Private m_ToolStripBorder As AHSLColor
    Private m_ToolStripContentPanelGradientBegin As AHSLColor
    Private m_ToolStripContentPanelGradientEnd As AHSLColor
    Private m_ToolStripDropDownBackground As AHSLColor
    Private m_ToolStripGradientBegin As AHSLColor
    Private m_ToolStripGradientEnd As AHSLColor
    Private m_ToolStripGradientMiddle As AHSLColor
    Private m_ToolStripPanelGradientBegin As AHSLColor
    Private m_ToolStripPanelGradientEnd As AHSLColor
#End Region

#Region " Konstruktoren "

    Public Sub New()
        ' benannte Farben der ColorTable neu lesen
        GetColors_First()
        ' Anzeige-Farben festlegen
        GetColors()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt den Basis-Farbton fr die einfache Control-Hintergrundfarbe zurck oder legt diesen fest
    ''' </summary>
    ''' <remarks>Wert zwischen 0 und 1</remarks>
    Property ControlHue() As Double
        Get
            Return m_ControlHue
        End Get
        Set(ByVal value As Double)
            Dim NewHue As Double = Math.Max(Math.Min(value, 1.0R), 0.0R)
            If m_ControlHue <> NewHue Then
                ' neue Werte bernehmen
                m_ControlHue = NewHue
                ' Farbwerte der einzelnen Farben neu bestimmen
                GetColors()
                ' nderungs-Event liefern
                RaiseEvent Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt den Dehnungsfaktor fr die Farben untereinander zurck oder legt diesen fest
    ''' </summary>
    ''' <remarks>Wert zwischen -1 und +1</remarks>
    Property StretchFactor() As Decimal
        Get
            Return m_StretchFactor
        End Get
        Set(ByVal value As Decimal)
            Dim NewStretchFactor As Decimal = Math.Max(Math.Min(value, 1D), -1D)
            If m_StretchFactor <> NewStretchFactor Then
                ' neue Werte bernehmen
                m_StretchFactor = NewStretchFactor
                ' Farbwerte der einzelnen Farben neu bestimmen
                GetColors()
                ' nderungs-Event liefern
                RaiseEvent Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt die als Basistabelle verwendete ProfessionalColorTable zurck oder legt diese fest
    ''' </summary>
    Property BaseTable() As ProfessionalColorTable
        Get
            Return m_BaseTable
        End Get
        Set(ByVal value As ProfessionalColorTable)
            If m_BaseTable IsNot value AndAlso value IsNot Nothing Then
                ' neue Tabelle bernehmen
                m_BaseTable = value
                ' benannte Farben der ColorTable neu lesen
                GetColors_First()
                ' Anzeige-Farben festlegen
                GetColors()
            End If
        End Set
    End Property

    Public Overrides ReadOnly Property ButtonCheckedGradientBegin() As System.Drawing.Color
        Get
            Return m_ButtonCheckedGradientBegin.DisplayColor
        End Get
    End Property
     _
    Public Overrides ReadOnly Property ButtonCheckedGradientEnd() As System.Drawing.Color
        Get
            Return m_ButtonCheckedGradientEnd.DisplayColor
        End Get
    End Property
     _
    Public Overrides ReadOnly Property ButtonCheckedGradientMiddle() As System.Drawing.Color
        Get
            Return m_ButtonCheckedGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonCheckedHighlight() As System.Drawing.Color
        Get
            Return m_ButtonCheckedHighlight.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonCheckedHighlightBorder() As System.Drawing.Color
        Get
            Return m_ButtonCheckedHighlightBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedBorder() As System.Drawing.Color
        Get
            Return m_ButtonPressedBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedGradientBegin() As System.Drawing.Color
        Get
            Return m_ButtonPressedGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedGradientEnd() As System.Drawing.Color
        Get
            Return m_ButtonPressedGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedGradientMiddle() As System.Drawing.Color
        Get
            Return m_ButtonPressedGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedHighlight() As System.Drawing.Color
        Get
            Return m_ButtonPressedHighlight.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonPressedHighlightBorder() As System.Drawing.Color
        Get
            Return m_ButtonPressedHighlightBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedBorder() As System.Drawing.Color
        Get
            Return m_ButtonSelectedBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedGradientBegin() As System.Drawing.Color
        Get
            Return m_ButtonSelectedGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedGradientEnd() As System.Drawing.Color
        Get
            Return m_ButtonSelectedGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedGradientMiddle() As System.Drawing.Color
        Get
            Return m_ButtonSelectedGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedHighlight() As System.Drawing.Color
        Get
            Return m_ButtonSelectedHighlight.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ButtonSelectedHighlightBorder() As System.Drawing.Color
        Get
            Return m_ButtonSelectedHighlightBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property CheckBackground() As System.Drawing.Color
        Get
            Return m_CheckBackground.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property CheckPressedBackground() As System.Drawing.Color
        Get
            Return m_CheckPressedBackground.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property CheckSelectedBackground() As System.Drawing.Color
        Get
            Return m_CheckSelectedBackground.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property GripDark() As System.Drawing.Color
        Get
            Return m_GripDark.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property GripLight() As System.Drawing.Color
        Get
            Return m_GripLight.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginGradientBegin() As System.Drawing.Color
        Get
            Return m_ImageMarginGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginGradientEnd() As System.Drawing.Color
        Get
            Return m_ImageMarginGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginGradientMiddle() As System.Drawing.Color
        Get
            Return m_ImageMarginGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginRevealedGradientBegin() As System.Drawing.Color
        Get
            Return m_ImageMarginRevealedGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginRevealedGradientEnd() As System.Drawing.Color
        Get
            Return m_ImageMarginRevealedGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ImageMarginRevealedGradientMiddle() As System.Drawing.Color
        Get
            Return m_ImageMarginRevealedGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuBorder() As System.Drawing.Color
        Get
            Return m_MenuBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemBorder() As System.Drawing.Color
        Get
            Return m_MenuItemBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemPressedGradientBegin() As System.Drawing.Color
        Get
            Return m_MenuItemPressedGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemPressedGradientEnd() As System.Drawing.Color
        Get
            Return m_MenuItemPressedGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemPressedGradientMiddle() As System.Drawing.Color
        Get
            Return m_MenuItemPressedGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemSelected() As System.Drawing.Color
        Get
            Return m_MenuItemSelected.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemSelectedGradientBegin() As System.Drawing.Color
        Get
            Return m_MenuItemSelectedGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuItemSelectedGradientEnd() As System.Drawing.Color
        Get
            Return m_MenuItemSelectedGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuStripGradientBegin() As System.Drawing.Color
        Get
            Return m_MenuStripGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property MenuStripGradientEnd() As System.Drawing.Color
        Get
            Return m_MenuStripGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property OverflowButtonGradientBegin() As System.Drawing.Color
        Get
            Return m_OverflowButtonGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property OverflowButtonGradientEnd() As System.Drawing.Color
        Get
            Return m_OverflowButtonGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property OverflowButtonGradientMiddle() As System.Drawing.Color
        Get
            Return m_OverflowButtonGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property RaftingContainerGradientBegin() As System.Drawing.Color
        Get
            Return m_RaftingContainerGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property RaftingContainerGradientEnd() As System.Drawing.Color
        Get
            Return m_RaftingContainerGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property SeparatorDark() As System.Drawing.Color
        Get
            Return m_SeparatorDark.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property SeparatorLight() As System.Drawing.Color
        Get
            Return m_SeparatorLight.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property StatusStripGradientBegin() As System.Drawing.Color
        Get
            Return m_StatusStripGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property StatusStripGradientEnd() As System.Drawing.Color
        Get
            Return m_StatusStripGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripBorder() As System.Drawing.Color
        Get
            Return m_ToolStripBorder.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripContentPanelGradientBegin() As System.Drawing.Color
        Get
            Return m_ToolStripContentPanelGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripContentPanelGradientEnd() As System.Drawing.Color
        Get
            Return m_ToolStripContentPanelGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripDropDownBackground() As System.Drawing.Color
        Get
            Return m_ToolStripDropDownBackground.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripGradientBegin() As System.Drawing.Color
        Get
            Return m_ToolStripGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripGradientEnd() As System.Drawing.Color
        Get
            Return m_ToolStripGradientEnd.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripGradientMiddle() As System.Drawing.Color
        Get
            Return m_ToolStripGradientMiddle.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripPanelGradientBegin() As System.Drawing.Color
        Get
            Return m_ToolStripPanelGradientBegin.DisplayColor
        End Get
    End Property
    Public Overrides ReadOnly Property ToolStripPanelGradientEnd() As System.Drawing.Color
        Get
            Return m_ToolStripPanelGradientEnd.DisplayColor
        End Get
    End Property
#End Region

#Region " Methoden "

    ''' <summary>
    ''' legt Basis-Farbton und Dehnungsfaktor fr die abgewandelte Farbtabelle neu fest
    ''' </summary>
    ''' <param name="ControlHue">Farbton fr die einfache Hintergrundfarbe fr Controls: Ein Wert zwischen 0 und 1</param>
    ''' <param name="StretchFactor">Dehnungsfaktor fr die Farben untereinander: Ein Wert zwischen -1 und 1</param>
    Public Sub SetHueAndStretchFactor(ByVal ControlHue As Double, ByVal StretchFactor As Decimal)
        Dim NewHue As Double = Math.Max(Math.Min(ControlHue, 1.0R), 0.0R)
        Dim NewStretchFactor As Decimal = Math.Max(Math.Min(StretchFactor, 1D), -1D)
        If m_ControlHue <> NewHue OrElse m_StretchFactor <> NewStretchFactor Then
            ' neue Werte bernehmen
            m_ControlHue = NewHue
            m_StretchFactor = NewStretchFactor
            ' Farbwerte der einzelnen Farben neu bestimmen
            GetColors()
            ' nderungs-Event liefern
            RaiseEvent Changed()
        End If
    End Sub

    ''' <summary>
    ''' liest alle benannten Farben einer neuen Farbtabelle ein, bestimmt
    ''' Mittelwert und Farbwert-Dehnungsfaktor fr diese Farben untereinander
    ''' </summary>
    Private Sub GetColors_First()
        m_ButtonCheckedGradientBegin = New AHSLColor(m_BaseTable.ButtonCheckedGradientBegin)
        m_ButtonCheckedGradientEnd = New AHSLColor(m_BaseTable.ButtonCheckedGradientEnd)
        m_ButtonCheckedGradientMiddle = New AHSLColor(m_BaseTable.ButtonCheckedGradientMiddle)
        m_ButtonCheckedHighlight = New AHSLColor(m_BaseTable.ButtonCheckedHighlight)
        m_ButtonCheckedHighlightBorder = New AHSLColor(m_BaseTable.ButtonCheckedHighlightBorder)
        m_ButtonPressedBorder = New AHSLColor(m_BaseTable.ButtonPressedBorder)
        m_ButtonPressedGradientBegin = New AHSLColor(m_BaseTable.ButtonPressedGradientBegin)
        m_ButtonPressedGradientEnd = New AHSLColor(m_BaseTable.ButtonPressedGradientEnd)
        m_ButtonPressedGradientMiddle = New AHSLColor(m_BaseTable.ButtonPressedGradientMiddle)
        m_ButtonPressedHighlight = New AHSLColor(m_BaseTable.ButtonPressedHighlight)
        m_ButtonPressedHighlightBorder = New AHSLColor(m_BaseTable.ButtonPressedHighlightBorder)
        m_ButtonSelectedBorder = New AHSLColor(m_BaseTable.ButtonSelectedBorder)
        m_ButtonSelectedGradientBegin = New AHSLColor(m_BaseTable.ButtonSelectedGradientBegin)
        m_ButtonSelectedGradientEnd = New AHSLColor(m_BaseTable.ButtonSelectedGradientEnd)
        m_ButtonSelectedGradientMiddle = New AHSLColor(m_BaseTable.ButtonSelectedGradientMiddle)
        m_ButtonSelectedHighlight = New AHSLColor(m_BaseTable.ButtonSelectedHighlight)
        m_ButtonSelectedHighlightBorder = New AHSLColor(m_BaseTable.ButtonSelectedHighlightBorder)
        m_CheckBackground = New AHSLColor(m_BaseTable.CheckBackground)
        m_CheckPressedBackground = New AHSLColor(m_BaseTable.CheckPressedBackground)
        m_CheckSelectedBackground = New AHSLColor(m_BaseTable.CheckSelectedBackground)
        m_GripDark = New AHSLColor(m_BaseTable.GripDark)
        m_GripLight = New AHSLColor(m_BaseTable.GripLight)
        m_ImageMarginGradientBegin = New AHSLColor(m_BaseTable.ImageMarginGradientBegin)
        m_ImageMarginGradientEnd = New AHSLColor(m_BaseTable.ImageMarginGradientEnd)
        m_ImageMarginGradientMiddle = New AHSLColor(m_BaseTable.ImageMarginGradientMiddle)
        m_ImageMarginRevealedGradientBegin = New AHSLColor(m_BaseTable.ImageMarginRevealedGradientBegin)
        m_ImageMarginRevealedGradientEnd = New AHSLColor(m_BaseTable.ImageMarginRevealedGradientEnd)
        m_ImageMarginRevealedGradientMiddle = New AHSLColor(m_BaseTable.ImageMarginRevealedGradientMiddle)
        m_MenuBorder = New AHSLColor(m_BaseTable.MenuBorder)
        m_MenuItemBorder = New AHSLColor(m_BaseTable.MenuItemBorder)
        m_MenuItemPressedGradientBegin = New AHSLColor(m_BaseTable.MenuItemPressedGradientBegin)
        m_MenuItemPressedGradientEnd = New AHSLColor(m_BaseTable.MenuItemPressedGradientEnd)
        m_MenuItemPressedGradientMiddle = New AHSLColor(m_BaseTable.MenuItemPressedGradientMiddle)
        m_MenuItemSelected = New AHSLColor(m_BaseTable.MenuItemSelected)
        m_MenuItemSelectedGradientBegin = New AHSLColor(m_BaseTable.MenuItemSelectedGradientBegin)
        m_MenuItemSelectedGradientEnd = New AHSLColor(m_BaseTable.MenuItemSelectedGradientEnd)
        m_MenuStripGradientBegin = New AHSLColor(m_BaseTable.MenuStripGradientBegin)
        m_MenuStripGradientEnd = New AHSLColor(m_BaseTable.MenuStripGradientEnd)
        m_OverflowButtonGradientBegin = New AHSLColor(m_BaseTable.OverflowButtonGradientBegin)
        m_OverflowButtonGradientEnd = New AHSLColor(m_BaseTable.OverflowButtonGradientEnd)
        m_OverflowButtonGradientMiddle = New AHSLColor(m_BaseTable.OverflowButtonGradientMiddle)
        m_RaftingContainerGradientBegin = New AHSLColor(m_BaseTable.RaftingContainerGradientBegin)
        m_RaftingContainerGradientEnd = New AHSLColor(m_BaseTable.RaftingContainerGradientEnd)
        m_SeparatorDark = New AHSLColor(m_BaseTable.SeparatorDark)
        m_SeparatorLight = New AHSLColor(m_BaseTable.SeparatorLight)
        m_StatusStripGradientBegin = New AHSLColor(m_BaseTable.StatusStripGradientBegin)
        m_StatusStripGradientEnd = New AHSLColor(m_BaseTable.StatusStripGradientEnd)
        m_ToolStripBorder = New AHSLColor(m_BaseTable.ToolStripBorder)
        m_ToolStripContentPanelGradientBegin = New AHSLColor(m_BaseTable.ToolStripContentPanelGradientBegin)
        m_ToolStripContentPanelGradientEnd = New AHSLColor(m_BaseTable.ToolStripContentPanelGradientEnd)
        m_ToolStripDropDownBackground = New AHSLColor(m_BaseTable.ToolStripDropDownBackground)
        m_ToolStripGradientBegin = New AHSLColor(m_BaseTable.ToolStripGradientBegin)
        m_ToolStripGradientEnd = New AHSLColor(m_BaseTable.ToolStripGradientEnd)
        m_ToolStripGradientMiddle = New AHSLColor(m_BaseTable.ToolStripGradientMiddle)
        m_ToolStripPanelGradientBegin = New AHSLColor(m_BaseTable.ToolStripPanelGradientBegin)
        m_ToolStripPanelGradientEnd = New AHSLColor(m_BaseTable.ToolStripPanelGradientEnd)

        m_AllColors.Clear()

        m_AllColors.Add(m_ButtonCheckedGradientBegin)
        m_AllColors.Add(m_ButtonCheckedGradientEnd)
        m_AllColors.Add(m_ButtonCheckedGradientMiddle)
        m_AllColors.Add(m_ButtonCheckedHighlight)
        m_AllColors.Add(m_ButtonCheckedHighlightBorder)
        m_AllColors.Add(m_ButtonPressedBorder)
        m_AllColors.Add(m_ButtonPressedGradientBegin)
        m_AllColors.Add(m_ButtonPressedGradientEnd)
        m_AllColors.Add(m_ButtonPressedGradientMiddle)
        m_AllColors.Add(m_ButtonPressedHighlight)
        m_AllColors.Add(m_ButtonPressedHighlightBorder)
        m_AllColors.Add(m_ButtonSelectedBorder)
        m_AllColors.Add(m_ButtonSelectedGradientBegin)
        m_AllColors.Add(m_ButtonSelectedGradientEnd)
        m_AllColors.Add(m_ButtonSelectedGradientMiddle)
        m_AllColors.Add(m_ButtonSelectedHighlight)
        m_AllColors.Add(m_ButtonSelectedHighlightBorder)
        m_AllColors.Add(m_CheckBackground)
        m_AllColors.Add(m_CheckPressedBackground)
        m_AllColors.Add(m_CheckSelectedBackground)
        m_AllColors.Add(m_GripDark)
        m_AllColors.Add(m_GripLight)
        m_AllColors.Add(m_ImageMarginGradientBegin)
        m_AllColors.Add(m_ImageMarginGradientEnd)
        m_AllColors.Add(m_ImageMarginGradientMiddle)
        m_AllColors.Add(m_ImageMarginRevealedGradientBegin)
        m_AllColors.Add(m_ImageMarginRevealedGradientEnd)
        m_AllColors.Add(m_ImageMarginRevealedGradientMiddle)
        m_AllColors.Add(m_MenuBorder)
        m_AllColors.Add(m_MenuItemBorder)
        m_AllColors.Add(m_MenuItemPressedGradientBegin)
        m_AllColors.Add(m_MenuItemPressedGradientEnd)
        m_AllColors.Add(m_MenuItemPressedGradientMiddle)
        m_AllColors.Add(m_MenuItemSelected)
        m_AllColors.Add(m_MenuItemSelectedGradientBegin)
        m_AllColors.Add(m_MenuItemSelectedGradientEnd)
        m_AllColors.Add(m_MenuStripGradientBegin)
        m_AllColors.Add(m_MenuStripGradientEnd)
        m_AllColors.Add(m_OverflowButtonGradientBegin)
        m_AllColors.Add(m_OverflowButtonGradientEnd)
        m_AllColors.Add(m_OverflowButtonGradientMiddle)
        m_AllColors.Add(m_RaftingContainerGradientBegin)
        m_AllColors.Add(m_RaftingContainerGradientEnd)
        m_AllColors.Add(m_SeparatorDark)
        m_AllColors.Add(m_SeparatorLight)
        m_AllColors.Add(m_StatusStripGradientBegin)
        m_AllColors.Add(m_StatusStripGradientEnd)
        m_AllColors.Add(m_ToolStripBorder)
        m_AllColors.Add(m_ToolStripContentPanelGradientBegin)
        m_AllColors.Add(m_ToolStripContentPanelGradientEnd)
        m_AllColors.Add(m_ToolStripDropDownBackground)
        m_AllColors.Add(m_ToolStripGradientBegin)
        m_AllColors.Add(m_ToolStripGradientEnd)
        m_AllColors.Add(m_ToolStripGradientMiddle)
        m_AllColors.Add(m_ToolStripPanelGradientBegin)
        m_AllColors.Add(m_ToolStripPanelGradientEnd)


        ' Minimalwert fr Hue
        Dim MinHue As Double = Double.MaxValue
        ' Maximalwert fr Hue
        Dim MaxHue As Double = Double.MinValue

        ' Minimum / Maximum bestimmen
        For Each col As AHSLColor In m_AllColors
            If col.Hue < MinHue Then
                MinHue = col.Hue
            End If
            If col.Hue > MaxHue Then
                MaxHue = col.Hue
            End If
        Next

        ' Mittelwert fr Hue
        m_MidHue = CDec(MaxHue + MinHue) / 2D
        ' Dehnungswert fr alle Hue-Werte untereinander
        m_StretchFactor = CDec(MaxHue - MinHue)

        ' fr alle AHSLColor-Objekte den Hue-Abstand vom Mittelwert bestimmen
        For Each col As AHSLColor In m_AllColors
            col.CalculateHueDistance(m_MidHue, m_StretchFactor)
        Next

        ' als Control-Hintergrundfarbe verwende ich ToolStripGradientMiddle
        m_ControlHue = m_ToolStripGradientMiddle.Hue
    End Sub

    ''' <summary>
    ''' berechnet alle Farben fr angegebenen Farbwert und Dehnung neu
    ''' </summary>
    Private Sub GetColors()
        ' Multiplikation mit aktuellem StretchFactor nur fr 
        ' ToolStripGradientMiddle ausfhren
        m_ToolStripGradientMiddle.GetStretchedColor(m_MidHue, m_StretchFactor)

        ' Verschiebung fr den Hue-Wert fr ToolStripGradientMiddle nach Dehnung ermitteln
        Dim HueShift As Decimal = CDec(m_ToolStripGradientMiddle.StretchedHue - m_ControlHue)
        ' Diese Verschiebung gilt auch fr alle anderen Basisfarben
        Dim NewMidHue As Decimal = m_MidHue - HueShift
        For Each col As AHSLColor In m_AllColors
            col.GetStretchedColor(NewMidHue, m_StretchFactor)
        Next

    End Sub

#End Region

End Class
