Imports System.Xml
''' <summary>
''' Repesents a single address.
''' </summary>
Public Class Address

#Region " Fields / Events "
    ''' <summary>
    ''' Resource for language specific string functions.
    ''' </summary>
    Private Shared m_LS As LanguageStrings

    Private m_CompanyName As String = ""
    Private m_Department As String = ""
    Private m_TitleInCompany As String = ""
    Private m_Title As String = ""
    Private m_FirstName As String = ""
    Private m_LastName As String = ""
    Private m_Gender As Genders = Genders.Male
    Private m_Country As String = ""
    Private m_Region As String = ""
    Private m_PostalCode As String = ""
    Private m_Street As String = ""
    Private m_City As String = ""
    Private m_Phone As String = ""
    Private m_Fax As String = ""
    Private m_HomePage As String = ""
    Private m_BirthDate As Date = Date.MinValue
    Private m_Abbreviation As String = ""

    Private m_Salutation As String = ""
    Private m_Salutation_Manually_Changed As Boolean

    Private m_Closing As String = ""
    Private m_Closing_Manually_Changed As Boolean

    Public Event Abbreviation_Changed()
    Public Event Salutation_Changed()
    Public Event Closing_Changed()
#End Region

#Region " Constructor "
    Shared Sub New()
        m_LS = LanguageStrings.Create
    End Sub
    ''' <summary>
    ''' Creates an empty address.
    ''' </summary>
    Public Sub New()
        Create_Abbreviation()
        Create_Salutation()
        Create_Closing()
    End Sub
    ''' <summary>
    ''' Creates a new address from XML node and corrects language specific content.
    ''' </summary>
    Public Sub New(ByVal Node As XmlNode, ByVal IsFromTemplate As Boolean)
        Me.New(Node)
        Create_Abbreviation()
        Create_Salutation()
        Create_Closing()
    End Sub
    ''' <summary>
    ''' Reads an address from XML node.
    ''' </summary>
    Public Sub New(ByVal Node As XmlNode)
        m_CompanyName = Node.Attributes("CompanyName").Value
        m_Department = Node.Attributes("Department").Value
        m_TitleInCompany = Node.Attributes("TitleInCompany").Value
        m_Title = Node.Attributes("Title").Value
        m_FirstName = Node.Attributes("FirstName").Value
        m_LastName = Node.Attributes("LastName").Value

        m_Country = Node.Attributes("Country").Value
        m_Region = Node.Attributes("Region").Value
        m_PostalCode = Node.Attributes("PostalCode").Value
        m_Street = Node.Attributes("Street").Value
        m_City = Node.Attributes("City").Value
        m_Phone = Node.Attributes("Phone").Value
        m_Fax = Node.Attributes("Fax").Value
        m_HomePage = Node.Attributes("HomePage").Value

        m_Abbreviation = Node.Attributes("Abbreviation").Value
        m_Salutation = Node.Attributes("Salutation").Value
        m_Closing = Node.Attributes("Closing").Value

        m_Salutation_Manually_Changed = CBool(Node.Attributes("Salutation_Manually_Changed").Value)
        m_Closing_Manually_Changed = CBool(Node.Attributes("Closing_Manually_Changed").Value)

        m_BirthDate = New Date(CLng(Node.Attributes("BirthDate").Value))
        m_Gender = CType(CInt(Node.Attributes("Gender").Value), Genders)
    End Sub

#End Region

#Region " Properties "
    Property CompanyName() As String
        Get
            Return m_CompanyName
        End Get
        Set(ByVal value As String)
            If m_CompanyName <> value.Trim Then
                m_CompanyName = value.Trim
                Create_Abbreviation()
            End If
        End Set
    End Property
    Property Department() As String
        Get
            Return m_Department
        End Get
        Set(ByVal value As String)
            If m_Department <> value.Trim Then
                m_Department = value.Trim
                Create_Abbreviation()
            End If
        End Set
    End Property
    Property TitleInCompany() As String
        Get
            Return m_TitleInCompany
        End Get
        Set(ByVal value As String)
            If m_TitleInCompany <> value.Trim Then
                m_TitleInCompany = value.Trim
                Create_Abbreviation()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Title Of Courtesy
    ''' </summary>
    Property Title() As String
        Get
            Return m_Title
        End Get
        Set(ByVal value As String)
            If m_Title <> value.Trim Then
                m_Title = value.Trim
                Create_Salutation()
            End If
        End Set
    End Property
    Property FirstName() As String
        Get
            Return m_FirstName
        End Get
        Set(ByVal value As String)
            If m_FirstName <> value.Trim Then
                m_FirstName = value.Trim
                Create_Abbreviation()
            End If
        End Set
    End Property
    Property LastName() As String
        Get
            Return m_LastName
        End Get
        Set(ByVal value As String)
            If m_LastName <> value.Trim Then
                m_LastName = value.Trim
                Create_Abbreviation()
                Create_Salutation()
                Create_Closing()
            End If
        End Set
    End Property
    Property Gender() As Genders
        Get
            Return m_Gender
        End Get
        Set(ByVal value As Genders)
            If m_Gender <> value Then
                m_Gender = value
                Create_Salutation()
                Create_Closing()
            End If
        End Set
    End Property
    Property Country() As String
        Get
            Return m_Country
        End Get
        Set(ByVal value As String)
            If m_Country <> value.Trim Then
                m_Country = value.Trim
            End If
        End Set
    End Property
    Property Region() As String
        Get
            Return m_Region
        End Get
        Set(ByVal value As String)
            If m_Region <> value.Trim Then
                m_Region = value.Trim
            End If
        End Set
    End Property
    Property PostalCode() As String
        Get
            Return m_PostalCode
        End Get
        Set(ByVal value As String)
            If m_PostalCode <> value.Trim Then
                m_PostalCode = value.Trim
            End If
        End Set
    End Property
    Property Street() As String
        Get
            Return m_Street
        End Get
        Set(ByVal value As String)
            If m_Street <> value.Trim Then
                m_Street = value.Trim
            End If
        End Set
    End Property
    Property City() As String
        Get
            Return m_City
        End Get
        Set(ByVal value As String)
            If m_City <> value.Trim Then
                m_City = value.Trim
                Create_Abbreviation()
            End If
        End Set
    End Property
    Property Phone() As String
        Get
            Return m_Phone
        End Get
        Set(ByVal value As String)
            If m_Phone <> value.Trim Then
                m_Phone = value.Trim
            End If
        End Set
    End Property
    Property Fax() As String
        Get
            Return m_Fax
        End Get
        Set(ByVal value As String)
            If m_Fax <> value.Trim Then
                m_Fax = value.Trim
            End If
        End Set
    End Property
    Property HomePage() As String
        Get
            Return m_HomePage
        End Get
        Set(ByVal value As String)
            If m_HomePage <> value.Trim Then
                m_HomePage = value.Trim
            End If
        End Set
    End Property
    Property BirthDate() As Date
        Get
            Return m_BirthDate
        End Get
        Set(ByVal value As Date)
            If m_BirthDate <> value Then
                m_BirthDate = value
            End If
        End Set
    End Property
    Property Abbreviation() As String
        Get
            Return m_Abbreviation
        End Get
        Set(ByVal value As String)
            If m_Abbreviation <> value.Trim Then
                m_Abbreviation = value.Trim
                RaiseEvent Abbreviation_Changed()
            End If
        End Set
    End Property
    Property Salutation() As String
        Get
            Return m_Salutation
        End Get
        Set(ByVal value As String)
            If m_Salutation <> value.Trim Then
                m_Salutation = value.Trim
                m_Salutation_Manually_Changed = True
            End If
        End Set
    End Property
    Property Closing() As String
        Get
            Return m_Closing
        End Get
        Set(ByVal value As String)
            If m_Closing <> value.Trim Then
                m_Closing = value.Trim
                m_Closing_Manually_Changed = True
            End If
        End Set
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' Writes this address into an XML document.
    ''' </summary>
    Public Sub WriteXML(ByVal Writer As XmlTextWriter)

        Writer.WriteAttributeString("CompanyName", m_CompanyName)
        Writer.WriteAttributeString("Department", m_Department)
        Writer.WriteAttributeString("TitleInCompany", m_TitleInCompany)
        Writer.WriteAttributeString("Title", m_Title)
        Writer.WriteAttributeString("FirstName", m_FirstName)
        Writer.WriteAttributeString("LastName", m_LastName)
        Writer.WriteAttributeString("Country", m_Country)
        Writer.WriteAttributeString("Region", m_Region)
        Writer.WriteAttributeString("PostalCode", m_PostalCode)
        Writer.WriteAttributeString("Street", m_Street)
        Writer.WriteAttributeString("City", m_City)
        Writer.WriteAttributeString("Phone", m_Phone)
        Writer.WriteAttributeString("Fax", m_Fax)
        Writer.WriteAttributeString("HomePage", m_HomePage)
        Writer.WriteAttributeString("Abbreviation", m_Abbreviation)
        Writer.WriteAttributeString("Salutation", m_Salutation)
        Writer.WriteAttributeString("Closing", m_Closing)

        Writer.WriteAttributeString("Salutation_Manually_Changed", m_Salutation_Manually_Changed.ToString)
        Writer.WriteAttributeString("Closing_Manually_Changed", m_Closing_Manually_Changed.ToString)
        Writer.WriteAttributeString("BirthDate", m_BirthDate.Ticks.ToString)
        Writer.WriteAttributeString("Gender", CInt(m_Gender).ToString)
    End Sub
    ''' <summary>
    ''' Creates an abbreviation for this address.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Create_Abbreviation()
        Dim Result As String = m_LastName

        If m_LastName <> "" Then
            Result = m_LastName
            If m_FirstName <> "" Then Result &= " " & m_FirstName
            If m_CompanyName <> "" Then Result &= " " & m_CompanyName
        Else
            If m_CompanyName <> "" Then Result = m_CompanyName
            If m_Department <> "" Then Result &= " " & m_Department
            If m_TitleInCompany <> "" Then Result &= " " & m_TitleInCompany
        End If


        If m_City <> "" Then
            If Result <> "" Then
                Dim Parts() As String = Result.Split(" "c)
                Dim CityFound As Boolean = False
                For Each txt As String In Parts
                    If txt = m_City Then
                        CityFound = True
                        Exit For
                    End If
                Next
                If Not CityFound Then
                    Result &= " " & m_City
                End If
            Else
                Result = m_City
            End If
        End If

        If Result <> "" Then
            Abbreviation = Result.Trim
        Else
            Abbreviation = m_LS.GetTranslation("Empty Address")
        End If

    End Sub
    ''' <summary>
    ''' Recreates the salutation for this address.
    ''' </summary>
    Private Sub Create_Salutation()
        If m_Salutation_Manually_Changed Then Exit Sub

        Dim NewSalutation As String = m_LS.GetSalutation(m_Gender, m_LastName, m_Title)

        If m_Salutation <> NewSalutation Then
            m_Salutation = NewSalutation
            RaiseEvent Salutation_Changed()
        End If

    End Sub
    ''' <summary>
    ''' Recreates the closing for this address.
    ''' </summary>
    Private Sub Create_Closing()
        If m_Closing_Manually_Changed Then Exit Sub

        Dim NewClosing As String = m_LS.GetLetterClosing(m_Gender, m_LastName)
        If m_Closing <> NewClosing Then
            m_Closing = NewClosing
            RaiseEvent Closing_Changed()
        End If
    End Sub

#End Region

End Class
