Imports System.Xml
''' <summary>
''' Contains the global used list of addresses.
''' </summary>
Public Class AddressSource

#Region " Fields "
    ''' <summary>
    ''' Path to save the address list.
    ''' </summary>
    Private Shared m_Path As String
    ''' <summary>
    ''' Address list.
    ''' </summary>
    Private Shared m_Addresses As AddressList
#End Region

#Region " Constructor "
    Shared Sub New()

        m_Path = Application.StartupPath & IO.Path.DirectorySeparatorChar & "Addresses.xml"

        If IO.File.Exists(m_Path) Then
            Dim Nd As XmlNode = MyXML.Reader.Read(m_Path)
            m_Addresses = New AddressList(Nd)
        Else
            Dim CultureName As String = My.Application.Culture.Name.ToLower
            Dim Basepath As String = Application.StartupPath & IO.Path.DirectorySeparatorChar
            If CultureName.StartsWith("de") Then
                Dim Nd As XmlNode = MyXML.Reader.Read(Basepath & "TemplateAddressesGerman.xml")
                m_Addresses = New AddressList(Nd, True)
            Else
                Dim Nd As XmlNode = MyXML.Reader.Read(Basepath & "TemplateAddressesEnglish.xml")
                m_Addresses = New AddressList(Nd, True)
            End If

        End If
        m_Addresses.Sort(New AddressComparer)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Returns all addresses as list.
    ''' </summary>
    Shared ReadOnly Property AllAddresses() As AddressList
        Get
            Return m_Addresses
        End Get
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' Saves the address list.
    ''' </summary>
    Shared Sub Save()
        Dim Writer As New MyXML.Writer(m_Path)
        m_Addresses.WriteXML(Writer.GetWriter)
        Writer.Dispose()
    End Sub
    ''' <summary>
    ''' Returns an address list without a single excluded item.
    ''' </summary>
    Public Shared Function Exclude(ByVal ExcludedAddress As Address) As AddressList
        Dim Result As New AddressList
        For Each Addr As Address In m_Addresses
            If Not Addr Is ExcludedAddress Then
                Result.Add(Addr)
            End If
        Next
        Return Result
    End Function
#End Region

#Region " Private Classes "
    ''' <summary>
    ''' Comparer to sort the address list.
    ''' </summary>
    Private Class AddressComparer
        Implements IComparer(Of Address)

        Public Function Compare(ByVal x As Address, ByVal y As Address) As Integer Implements System.Collections.Generic.IComparer(Of Address).Compare
            Return x.Abbreviation.CompareTo(y.Abbreviation)
        End Function
    End Class
#End Region

End Class
