Imports OwnWordInterop.Document
Public Class AmericanStrings
    Inherits BritishStrings

#Region " Methods "

#Region " Letter Methods "
    ''' <summary>
    ''' Creates a language specific range description how to fill in the date field in a letter.
    ''' </summary>
    Protected Overrides Function Letter_DateLine(ByVal Addr As Address) As Range
        Dim Rng As New Range

        Rng.Text.Add(Format(Now, "MMMM dd, yyyy"))

        Return Rng
    End Function
    ''' <summary>
    ''' Creates a range description how to build the senders address field in Word documents.
    ''' </summary>
    Protected Overrides Function Letter_SendersLogo(ByVal Addr As Address) As Range
        Dim Result As Range = MyBase.Letter_SendersLogo(Addr)
        Result.Tables(0).Alignment = WdRowAlignment.wdAlignRowLeft
        Return Result
    End Function
    ''' <summary>
    ''' Creates a basic structure for a buisness letter with bookmarks.
    ''' </summary>
    Protected Overrides Function Letter_BasicStructure() As OwnWordInterop.Document.Range
        Dim Rng As New Range

        Rng.Format.Alignment = WdParagraphAlignment.wdAlignParagraphLeft
        Rng.Format.LineSpacingRule = WdLineSpacing.wdLineSpaceSingle
        Rng.Format.SpaceAfter = 6
        Rng.Format.SpaceBefore = 0

        Rng.Bookmarks.Add("Date")
        Rng.Paragraphs.Add().Format.SpaceAfter = 0

        Rng.Bookmarks.Add("SendersAddress")

        Rng.Paragraphs.Add()
        Rng.Breaks.Add(WdBreakType.wdSectionBreakContinuous)

        Dim Tbl As Table = Rng.Tables.Add(2, 1)
        Tbl.Columns(0).PreferredWidth = Range.CentimetersToPoints(8)
        Tbl.AutoFitBehavior = WdAutoFitBehavior.wdAutoFitContent
        Tbl.Cell(0, 0).Range.Bookmarks.Add("RecipientAddress")
        Tbl.Cell(1, 0).Range.Bookmarks.Add("SendersAddressLine")
        Tbl.Cell(1, 0).Borders.TopBorder = WdLineStyle.wdLineStyleSingle

        Rng.Paragraphs.Add().Format.SpaceAfter = 6
        Rng.Paragraphs.Add()

        Rng.Paragraphs.Add()
        Rng.Bookmarks.Add("SubjectLine")
        Rng.Paragraphs.Add()
        Rng.Bookmarks.Add("Salutation")

        Rng.Paragraphs.Add()
        Rng.Bookmarks.Add("Body")
        Rng.Paragraphs.Add()
        Rng.Bookmarks.Add("Greeting")
        Rng.Paragraphs.Add()
        Rng.Paragraphs.Add()
        Rng.Bookmarks.Add("Signature")

        Return Rng
    End Function
#End Region

#Region " Common Methods "
    ''' <summary>
    ''' Creates a language specific salutation for letters.
    ''' </summary>
    Public Overrides Function GetSalutation(ByVal Gender As Genders, ByVal LastName As String, ByVal Title As String) As String

        If Gender = Genders.Impersonal Then
            Return "Ladies and Gentlemen:"
        ElseIf LastName = "" Then
            If Gender = Genders.Male Then
                Return "Gentlemen:"
            Else
                Return "Ladies:"
            End If
        Else
            If Title = "" Then
                If Gender = Genders.Male Then
                    Return "Dear Mr. " & LastName & ":"
                Else
                    Return "Dear Ms. " & LastName & ":"
                End If
            Else
                Return "Dear " & Title & " " & LastName & ":"
            End If
        End If

    End Function
    ''' <summary>
    ''' Creates a language specific closing for letters.
    ''' </summary>
    Public Overrides Function GetLetterClosing(ByVal Gender As Genders, ByVal LastName As String) As String
        Return "Sincerely,"
    End Function
#End Region

#End Region

End Class
