Imports OwnWordInterop.Document
''' <summary>
''' Source for language specific address and letter creation functions and Windows Forms messages.
''' </summary>
Public MustInherit Class LanguageStrings

#Region " Constructor "
    ''' <summary>
    ''' Constructor.
    ''' </summary>
    Protected Sub New()

    End Sub
    ''' <summary>
    ''' Returns a language specific LanguageStrings subclass.
    ''' </summary>
    Public Shared Function Create() As LanguageStrings
        Dim CultureName As String = My.Application.Culture.Name.ToLower
        If CultureName.StartsWith("de") Then
            Return New GermanStrings
        ElseIf CultureName.StartsWith("en-us") Then
            Return New AmericanStrings
        Else
            Return New BritishStrings
        End If
    End Function
#End Region

#Region " Methods "

#Region " Windows Forms messages "
    ''' <summary>
    ''' Returns a string to display if missing or not selected ListViewItem was found in frm_Main.
    ''' </summary>
    MustOverride Function Frm_MissingDocumentString(ByVal Var As Integer, ByVal IsTemplate As Boolean, ByVal NotExists As Boolean) As String
    ''' <summary>
    ''' Returns a string to display Toolstrip Button Hoover Information in frm_Main.
    ''' </summary>
    MustOverride Function Frm_HooverExplanation(ByVal Hooverindex As Integer) As String
    ''' <summary>
    ''' Returns a string to display a message after document creation in frm_Main.
    ''' </summary>
    MustOverride Function Frm_Message(ByVal Index As Integer) As String
#End Region

#Region " Letter Methods "
    ''' <summary>
    ''' Adds a paragraph line to an OwnWordInterop.Range.
    ''' </summary>
    Protected Shared Sub AddLine(ByVal Text As String, ByVal Rng As Range, ByRef WasSomeInserted As Boolean)
        If Text.Trim <> "" Then
            If WasSomeInserted Then
                Rng.Paragraphs.Add()
            End If
            WasSomeInserted = True
            Rng.Text.Add(Text.Trim)
        End If
    End Sub
    ''' <summary>
    ''' Fills an range description with address data of recipient to fill-in bookmarks in a Word template.
    ''' and to create a document later.
    ''' </summary>
    Function FillLetter(ByVal Recipient As Address, ByVal Sender As Address) As Range
        Dim Rng As New Range
        Rng.Bookmarks.Item("RecipientAddress").Range = Letter_Recipient(Recipient)
        Rng.Bookmarks.Item("Date").Range = Letter_DateLine(Sender)
        Rng.Bookmarks.Item("Salutation").Range.Text.Add(Recipient.Salutation)
        Rng.Bookmarks.Item("Greeting").Range.Text.Add(Recipient.Closing)
        Rng.Bookmarks.Item("Body").Selected = True
        Return Rng
    End Function
    ''' <summary>
    ''' Creates a description how to fill in the recipient address field in a letter.
    ''' </summary>
    Private Function Letter_Recipient(ByVal Addr As Address) As Range
        Dim Rng As New Range
        Dim Txt As String = ""
        Dim WasSomethingAdded As Boolean = False

        If Addr.FirstName <> "" AndAlso Addr.LastName <> "" Then
            If Addr.Title <> "" Then Txt &= Addr.Title & " "
            Txt &= Addr.FirstName & " " & Addr.LastName
            AddLine(Txt, Rng, WasSomethingAdded)
        End If

        AddLine(Addr.TitleInCompany, Rng, WasSomethingAdded)
        AddLine(Addr.CompanyName, Rng, WasSomethingAdded)
        AddLine(Addr.Street, Rng, WasSomethingAdded)
        AddLine(Letter_CityLine(Addr), Rng, WasSomethingAdded)
        AddLine(Addr.Country, Rng, WasSomethingAdded)

        Return Rng
    End Function
    ''' <summary>
    ''' Returns a language specific string with city, region and postalcode for letters.
    ''' </summary>
    Protected MustOverride Function Letter_CityLine(ByVal Addr As Address) As String
    ''' <summary>
    ''' Creates a language specific description how to fill in the date field in a letter.
    ''' </summary>
    Protected MustOverride Function Letter_DateLine(ByVal Addr As Address) As Range

    ''' <summary>
    ''' Fills an range description with address data of sender to create a Word template later.
    ''' </summary>
    Function FillTemplate(ByVal Addr As Address) As Range
        Dim Rng As Range = Letter_BasicStructure()


        Rng.Bookmarks.Item("SendersAddress").Range = Letter_SendersLogo(Addr)
        Rng.Bookmarks.Item("SendersAddressLine").Range = Letter_SendersLine(Addr)

        If Addr.FirstName <> "" AndAlso Addr.LastName <> "" Then
            Dim Signature As String = ""
            If Addr.Title <> "" Then
                Signature &= " " & Addr.Title
            End If
            Signature &= " " & Addr.FirstName & " " & Addr.LastName
            Rng.Bookmarks.Item("Signature").Range.Text.Add(Signature.Trim)
        End If

        Return Rng
    End Function
    ''' <summary>
    ''' Creates a basic structure for a buisness letter with bookmarks.
    ''' </summary>
    Protected MustOverride Function Letter_BasicStructure() As Range
    ''' <summary>
    ''' Creates a description how to build the senders address field in Word documents.
    ''' </summary>
    Protected Overridable Function Letter_SendersLogo(ByVal Addr As Address) As Range
        Dim Rng As New Range

        Dim Txt As String = ""
        Dim WasSomeInserted As Boolean = False

        Dim NumRows As Integer = 1
        If Addr.Phone <> "" Then NumRows += 1
        If Addr.Fax <> "" Then NumRows += 1
        Dim Tbl As Table = Rng.Tables.Add(NumRows, 2)

        Dim Rng1 As Range = Tbl.Cell(0, 0).Range

        If Addr.FirstName <> "" AndAlso Addr.LastName <> "" Then
            If Addr.Title <> "" Then Txt &= Addr.Title & " "
            Txt &= Addr.FirstName & " " & Addr.LastName
            AddLine(Txt, Rng1, WasSomeInserted)
        End If

        AddLine(Addr.TitleInCompany, Rng1, WasSomeInserted)
        AddLine(Addr.CompanyName, Rng1, WasSomeInserted)
        AddLine(Addr.Street, Rng1, WasSomeInserted)
        AddLine(Letter_CityLine(Addr).Trim, Rng1, WasSomeInserted)

        Dim RowIndex As Integer = 1
        If Addr.Phone <> "" Then
            Tbl.Cell(RowIndex, 0).Range.Text.Add(GetTranslation("Phone"))
            Tbl.Cell(RowIndex, 1).Range.Text.Add(Addr.Phone)
            RowIndex += 1
        End If
        If Addr.Fax <> "" Then
            Tbl.Cell(RowIndex, 0).Range.Text.Add(GetTranslation("Fax"))
            Tbl.Cell(RowIndex, 1).Range.Text.Add(Addr.Fax)
        End If

        Tbl.Cell(0, 0).MergeTo = Tbl.Cell(0, 1)
        Tbl.AutoFitBehavior = WdAutoFitBehavior.wdAutoFitContent

        Return Rng
    End Function

    ''' <summary>
    ''' Creates a small address line.
    ''' </summary>
    Private Function Letter_SendersLine(ByVal Addr As Address) As Range
        Dim Rng As New Range
        Dim Txt As String = ""
        Dim Segm As TextSegment

        If Addr.CompanyName <> "" Then
            Txt &= Addr.CompanyName & " " & Chr(149)
        ElseIf Addr.FirstName <> "" AndAlso Addr.LastName <> "" Then
            If Addr.Title <> "" Then Txt &= Addr.Title & " "
            Txt &= Addr.FirstName & " " & Addr.LastName & " " & Chr(149)
        End If

        If Addr.Street <> "" Then
            Txt &= " " & Addr.Street & " " & Chr(149)
        End If

        Txt &= Letter_CityLine(Addr)

        Txt = Txt.Trim
        If Txt <> "" Then
            Segm = Rng.Text.Add(Txt)
            Segm.Font.Size = 8
        End If
        Rng.Format.Alignment = WdParagraphAlignment.wdAlignParagraphCenter

        Return Rng
    End Function
#End Region

#Region " Common Methods "
    ''' <summary>
    ''' Creates a language specific salutation for letters.
    ''' </summary>
    MustOverride Function GetSalutation(ByVal Gender As Genders, ByVal LastName As String, ByVal Title As String) As String
    ''' <summary>
    ''' Creates a language specific closing for letters.
    ''' </summary>
    MustOverride Function GetLetterClosing(ByVal Gender As Genders, ByVal LastName As String) As String
    ''' <summary>
    ''' Returns a language specific string.
    ''' </summary>
    MustOverride Function GetTranslation(ByVal Source As String) As String
#End Region

#End Region

End Class
