''' <summary>
''' Windows form to select or edit the address of a letter recipient.
''' </summary>
Public Class frm_FillDocument

    ''' <summary>
    ''' Current selected address.
    ''' </summary>
    ReadOnly Property ActiveAddress() As Address
        Get
            Return CType(Me.AddressListBindingSource.Current, Address)
        End Get
    End Property

    Public Sub New()
        InitializeComponent()
    End Sub

    ''' <summary>
    ''' Constructor with list of addresses as data source and defined selected address.
    ''' </summary>
    Public Sub New(ByVal Addresses As AddressList, ByVal SelectedAddress As Address)
        InitializeComponent()

        Me.AddressListBindingSource.DataSource = Addresses
        Me.ComboBox1.DataSource = Me.AddressListBindingSource
        Me.ComboBox1.DisplayMember = "Abbreviation"
        Me.ComboBox1.ValueMember = "Abbreviation"
        Me.ComboBox1.SelectedItem = SelectedAddress
    End Sub


    ''' <summary>
    ''' Saves address changes and closes the form.
    ''' </summary>
    Private Sub ButtonOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonOK.Click
        Me.DialogResult = Windows.Forms.DialogResult.OK
        AddressSource.Save()
        Me.Close()
    End Sub

    ''' <summary>
    ''' Closes the windows form and stores no changes.
    ''' </summary>
    Private Sub ButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonCancel.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Private Sub frm_FillDocument_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        If My.Application.Culture.Name.ToLower.StartsWith("de") Then
            ButtonCancel.Text = My.Resources.GermanStrings.ButtonCancel_Text
            CityLabel.Text = My.Resources.GermanStrings.CityLabel_Text
            CompanyNameLabel.Text = My.Resources.GermanStrings.CompanyNameLabel_Text
            CountryLabel.Text = My.Resources.GermanStrings.CountryLabel_Text
            FirstNameLabel.Text = My.Resources.GermanStrings.FirstNameLabel_Text
            Label2.Text = My.Resources.GermanStrings.Label2_Text
            LastNameLabel.Text = My.Resources.GermanStrings.LastNameLabel_Text
            PostalCodeLabel.Text = My.Resources.GermanStrings.PostalCodeLabel_Text
            StreetLabel.Text = My.Resources.GermanStrings.StreetLabel_Text
            TitleInCompanyLabel.Text = My.Resources.GermanStrings.TitleInCompanyLabel_Text
            TitleLabel.Text = My.Resources.GermanStrings.TitleLabel_Text
            LabelDescription.Text = My.Resources.GermanStrings.LabelDescription_Text
            ClosingLabel.Text = My.Resources.GermanStrings.ClosingLabel_Text
            GenderLabel.Text = My.Resources.GermanStrings.GenderLabel_Text
            SalutationLabel.Text = My.Resources.GermanStrings.SalutationLabel_Text
            Me.Text = My.Resources.GermanStrings.frm_FillDocument_Text
        End If
    End Sub
End Class
