Imports OwnWordInterop
Imports OwnWordInterop.Document
''' <summary>
''' Main Form 
''' </summary>
Public Class frm_Main

#Region " Fields "
    ''' <summary>
    ''' Interacts with Word to create new documents.
    ''' </summary>
    Private WithEvents m_DocumentCreator As New OwnWordInterop.DocumentCreator
    ''' <summary>
    ''' Path for new Word documents.
    ''' </summary>
    Private m_DocumentsPath As String
    ''' <summary>
    ''' Path for new Word templates.
    ''' </summary>
    Private m_TemplatesPath As String
    ''' <summary>
    ''' Last for a template used address.
    ''' </summary>
    Private m_SelectedTemplateAddress As Address
    ''' <summary>
    ''' Index of hoovered ToolstripButton.
    ''' </summary>
    Private m_HooverIndex As Integer = -2
    ''' <summary>
    ''' Creates language-dependend strings.
    ''' </summary>
    Private m_LS As LanguageStrings
    ''' <summary>
    ''' Informs about the selection state in a ListView.
    ''' </summary>
    Private Enum SelectionValue
        NotExists
        NotSelected
        Selected
    End Enum
#End Region

#Region " Methods "
    ''' <summary>
    ''' Initializes imagelists, path and address information.
    ''' </summary>
    Private Sub frm_Main_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        m_DocumentsPath = Application.StartupPath & IO.Path.DirectorySeparatorChar & "Documents"
        m_TemplatesPath = Application.StartupPath & IO.Path.DirectorySeparatorChar & "Templates"

        ClearDirectory(m_DocumentsPath)
        ClearDirectory(m_TemplatesPath)

        If Not IO.Directory.Exists(m_DocumentsPath) Then
            IO.Directory.CreateDirectory(m_DocumentsPath)
        End If
        If Not IO.Directory.Exists(m_TemplatesPath) Then
            IO.Directory.CreateDirectory(m_TemplatesPath)
        End If

        Dim IL1 As New ImageList
        IL1.ColorDepth = ColorDepth.Depth32Bit
        IL1.ImageSize = New Size(16, 16)
        IL1.Images.Add(My.Resources.WordDot2000_16)
        Me.ListViewTemplates.SmallImageList = IL1

        Dim IL2 As New ImageList
        IL2.ColorDepth = ColorDepth.Depth32Bit
        IL2.ImageSize = New Size(16, 16)
        IL2.Images.Add(My.Resources.WordDoc2000_16)
        Me.ListViewDocuments.SmallImageList = IL2

        m_LS = LanguageStrings.Create
        m_SelectedTemplateAddress = AddressSource.AllAddresses(0)
        SetHooverIndex(-1)

        If My.Application.Culture.Name.ToLower.StartsWith("de") Then
            Bluelabel1.Text = My.Resources.GermanStrings.Bluelabel1_Text
            Bluelabel2.Text = My.Resources.GermanStrings.Bluelabel2_Text
            Label1.Text = My.Resources.GermanStrings.frm_main_Labeltext
            ToolStripButtonCreateTemplate.Text = My.Resources.GermanStrings.ToolStripButtonCreateTemplate_Text
            ToolStripButtonCreateTemplate.ToolTipText = My.Resources.GermanStrings.ToolStripButtonCreateTemplate_ToolTipText
            ToolStripButtonFillDocument.Text = My.Resources.GermanStrings.ToolStripButtonFillDocument_Text
            ToolStripButtonFillDocument.ToolTipText = My.Resources.GermanStrings.ToolStripButtonFillDocument_ToolTipText
            ToolStripButtonReadHidden.Text = My.Resources.GermanStrings.ToolStripButtonReadHidden_Text
            ToolStripButtonReadHidden.ToolTipText = My.Resources.GermanStrings.ToolStripButtonReadHidden_ToolTipText
            ToolStripButtonRefillDocument.Text = My.Resources.GermanStrings.ToolStripButtonRefillDocument_Text
            ToolStripButtonRefillDocument.ToolTipText = My.Resources.GermanStrings.ToolStripButtonRefillDocument_ToolTipText
            ToolStripButtonWriteHidden.Text = My.Resources.GermanStrings.ToolStripButtonWriteHidden_Text
            ToolStripButtonWriteHidden.ToolTipText = My.Resources.GermanStrings.ToolStripButtonWriteHidden_ToolTipText
        End If

    End Sub
    ''' <summary>
    ''' Clears document and template directories.
    ''' </summary>
    Private Sub frm_Main_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ClearDirectory(m_DocumentsPath)
        ClearDirectory(m_TemplatesPath)
    End Sub
    ''' <summary>
    ''' Removes documents and templates from a single directory.
    ''' </summary>
    Private Sub ClearDirectory(ByVal Path As String)
        If IO.Directory.Exists(Path) Then
            Dim Waserror As Boolean = False
            Try
                IO.Directory.Delete(Path)
            Catch ex As Exception
                Waserror = True
            End Try
            If Waserror Then
                Dim Files() As String = IO.Directory.GetFiles(Path)
                For Each File As String In Files
                    Try
                        IO.File.Delete(File)
                    Catch ex As Exception

                    End Try
                Next
            End If
        End If
    End Sub
    ''' <summary>
    ''' Checks if a secified ListView has selected items, has items or not.
    ''' </summary>
    Private Function GetListViewSelection(ByVal LV As ListView) As SelectionValue
        If LV.Items.Count = 0 Then
            Return SelectionValue.NotExists
        ElseIf LV.SelectedItems.Count = 0 Then
            Return SelectionValue.NotSelected
        Else
            Return SelectionValue.Selected
        End If
    End Function
    ''' <summary>
    ''' Checks for needed selection in specified ListView.
    ''' </summary>
    Private Function GetMissingSelection(ByVal ShowMessage As Boolean, ByVal IsTemplate As Boolean, Optional ByVal SpecificVariant As Integer = -1) As String
        Dim SelectionValue As SelectionValue
        If IsTemplate Then
            SelectionValue = GetListViewSelection(Me.ListViewTemplates)
        Else
            SelectionValue = GetListViewSelection(Me.ListViewDocuments)
        End If
        If SelectionValue = frm_Main.SelectionValue.Selected Then Return ""


        Dim Nmb As Integer
        If SpecificVariant <> -1 Then
            Nmb = SpecificVariant
        Else
            Randomize()
            Nmb = CInt(Int(2.0F * Rnd()))
        End If


        Dim Result As String = m_LS.Frm_MissingDocumentString(Nmb, IsTemplate, SelectionValue = SelectionValue.NotExists)


        If ShowMessage Then
            MessageBox.Show(Result, "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
        Return Result

    End Function
    ''' <summary>
    ''' Displays information in a label.
    ''' </summary>
    Private Sub SetHooverIndex(ByVal NewValue As Integer)
        If m_HooverIndex <> NewValue Then
            m_HooverIndex = NewValue

            Dim HooverText As String = m_LS.Frm_HooverExplanation(m_HooverIndex)
            Dim xExtratext As String = ""
            Select Case m_HooverIndex
                Case 1
                    xExtratext = GetMissingSelection(False, True, 0)
                Case 3
                    xExtratext = GetMissingSelection(False, False, 0)
                Case 2, 4
                    xExtratext = GetMissingSelection(False, False, 1)
            End Select
            If xExtratext <> "" Then
                xExtratext = " (" & xExtratext & ")"
            End If

            Me.LabelExplanation.Text = HooverText & xExtratext

        End If
    End Sub
    Private Sub ToolStripButtonCreateTemplate_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCreateTemplate.MouseEnter
        SetHooverIndex(0)
    End Sub
    Private Sub ToolStripButtonFillDocument_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonFillDocument.MouseEnter
        SetHooverIndex(1)
    End Sub
    Private Sub ToolStripButtonRefillDocument_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRefillDocument.MouseEnter
        SetHooverIndex(2)
    End Sub
    Private Sub ToolStripButtonWriteHidden_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonWriteHidden.MouseEnter
        SetHooverIndex(3)
    End Sub
    Private Sub ToolStripButtonReadHidden_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonReadHidden.MouseEnter
        SetHooverIndex(4)
    End Sub
    Private Sub ToolStrip1_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStrip1.MouseLeave
        SetHooverIndex(-1)
    End Sub
    Private Sub ToolStrip1_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStrip1.MouseEnter
        SetHooverIndex(-1)
    End Sub
    Private Sub ToolStripButtonCreateTemplate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCreateTemplate.Click
        CreateTemplate()
    End Sub
    Private Sub ToolStripButtonFillDocument_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonFillDocument.Click
        If GetMissingSelection(True, True) = "" Then
            CreateDocument()
        End If
    End Sub
    Private Sub ToolStripButtonRefillDocument_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRefillDocument.Click
        If GetMissingSelection(True, False) = "" Then
            CreateRefilledDocument()
        End If
    End Sub
    Private Sub ToolStripButtonWriteHidden_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonWriteHidden.Click
        If GetMissingSelection(True, False) = "" Then
            WriteHiddenText()
        End If
    End Sub
    Private Sub ToolStripButtonReadHidden_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonReadHidden.Click
        If GetMissingSelection(True, False) = "" Then
            ReadHiddenText()
        End If
    End Sub
    ''' <summary>
    ''' Creates a new Word template.
    ''' </summary>
    Private Sub CreateTemplate()

        Dim frm As New frm_FillTemplate(AddressSource.AllAddresses, m_SelectedTemplateAddress)
        If frm.ShowDialog <> Windows.Forms.DialogResult.OK Then Exit Sub
        m_SelectedTemplateAddress = frm.ActiveAddress
        If Not DocumentCreator.IsWordInstalled Then Exit Sub


        Dim Newpath As String = GetFreePath(True)
        Dim Addr As Address = frm.ActiveAddress


        Dim FillDescription As Range = m_LS.FillTemplate(Addr)
        Dim FirstPageTopMargin As Single = Range.CentimetersToPoints(1.5F)

        DocumentCreator.CreateDocument(FillDescription, Newpath, "", FirstPageTopMargin)

        Me.Activate()
        Me.Visible = False

        MessageBox.Show(m_LS.Frm_Message(0), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)

        Dim Itm As New TemplateListViewItem(Newpath, Addr)
        Me.ListViewTemplates.Items.Add(Itm)
        Me.ListViewTemplates.Select()
        Itm.Selected = True
        Me.Visible = True
    End Sub
    ''' <summary>
    ''' Fills a Word document based on a template and an address.
    ''' </summary>
    Private Sub CreateDocument()

        Dim LVI As TemplateListViewItem = DirectCast(Me.ListViewTemplates.SelectedItems(0), TemplateListViewItem)
        Dim TemplatePath As String = LVI.Path
        Dim TemplateAddress As Address = LVI.TemplateAddress

        Dim Adrs As AddressList = AddressSource.Exclude(TemplateAddress)
        Dim frm As New frm_FillDocument(Adrs, Adrs(0))
        If frm.ShowDialog <> Windows.Forms.DialogResult.OK Then Exit Sub
        If Not DocumentCreator.IsWordInstalled Then Exit Sub

        Dim Newpath As String = GetFreePath(False)
        Dim Addr As Address = frm.ActiveAddress

        Dim Rng As Range = m_LS.FillLetter(Addr, TemplateAddress)

        DocumentCreator.CreateDocument(Rng, Newpath, TemplatePath)

        Me.Activate()
        Me.Visible = False

        MessageBox.Show(m_LS.Frm_Message(1), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)

        Dim Itm As New DocumentListViewItem(Newpath, TemplateAddress, Addr)
        Me.ListViewDocuments.Items.Add(Itm)
        Me.ListViewDocuments.Select()
        Itm.Selected = True
        Me.Visible = True
    End Sub
    ''' <summary>
    ''' Fills in bookmarks in an existing Word document.
    ''' </summary>
    Private Sub CreateRefilledDocument()

        Dim LVI As DocumentListViewItem = DirectCast(Me.ListViewDocuments.SelectedItems(0), DocumentListViewItem)
        Dim DocumentPath As String = LVI.Path
        Dim TemplateAddress As Address = LVI.TemplateAddress
        Dim Adrs As AddressList = AddressSource.Exclude(TemplateAddress)

        Dim frm As New frm_FillDocument(Adrs, LVI.DocumentAddress)
        frm.Text = m_LS.Frm_Message(6)
        frm.LabelDescription.Text = m_LS.Frm_Message(7)
        If frm.ShowDialog <> Windows.Forms.DialogResult.OK Then Exit Sub

        Dim Addr As Address = frm.ActiveAddress
        LVI.DocumentAddress = Addr

        Dim Rng As Range = m_LS.FillLetter(Addr, TemplateAddress)

        DocumentCreator.CreateDocument(Rng, DocumentPath)

        Me.Activate()
        Me.Visible = False
        MessageBox.Show(m_LS.Frm_Message(2), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Me.ListViewDocuments.Select()
        Me.Visible = True
    End Sub
    ''' <summary>
    ''' Writes hidden text into a Word document.
    ''' </summary>
    Private Sub WriteHiddenText()
        Dim Txt As String = InputBox(m_LS.Frm_Message(8), "OwnWordRange Test", m_LS.Frm_Message(9)).Trim
        If Txt <> "" Then
            Dim LVI As DocumentListViewItem = DirectCast(Me.ListViewDocuments.SelectedItems(0), DocumentListViewItem)
            Dim DocumentPath As String = LVI.Path

            DocumentCreator.WriteHiddenText(DocumentPath, Txt)

            Me.Activate()
            Me.Visible = False
            MessageBox.Show(m_LS.Frm_Message(3), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)

            Me.Visible = True

        End If
    End Sub
    ''' <summary>
    ''' Reads hidden text from a Word document.
    ''' </summary>
    Private Sub ReadHiddenText()
        Dim LVI As DocumentListViewItem = DirectCast(Me.ListViewDocuments.SelectedItems(0), DocumentListViewItem)
        Dim DocumentPath As String = LVI.Path
        Dim Txt As String = DocumentCreator.ReadHiddenText(DocumentPath)
        If Txt = "" Then
            MessageBox.Show(m_LS.Frm_Message(4), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Else
            MessageBox.Show(m_LS.Frm_Message(5) & vbCrLf & vbCrLf & Chr(34) & Txt & Chr(34), "OwnWordRange Test", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub
    ''' <summary>
    ''' Finds a free path for a new document or template.
    ''' </summary>
    Private Function GetFreePath(ByVal Template As Boolean) As String
        Dim BasePath As String
        Dim Extension As String
        If Template Then
            BasePath = m_TemplatesPath & IO.Path.DirectorySeparatorChar & "Template"
            Extension = DocumentCreator.WordTemplateExtensions(0)
        Else
            BasePath = m_DocumentsPath & IO.Path.DirectorySeparatorChar & "TestDoc"
            Extension = DocumentCreator.WordDocumentExtensions(0)
        End If
        If Extension Is Nothing Then Return ""
        Dim Number As Integer = 1
        While IO.File.Exists(BasePath & Number.ToString & Extension)
            Number += 1
        End While
        Return BasePath & Number.ToString & Extension
    End Function
    ''' <summary>
    ''' Handler for the WordInstalled_Changed-Event of the DocumentCreator.
    ''' </summary>
    Private Sub m_DocumentCreator_WordInstalled_Changed() Handles m_DocumentCreator.IsWordInstalled_Changed
        If Not DocumentCreator.IsWordInstalled Then
            For Each ctl As ToolStripItem In Me.ToolStrip1.Items
                ctl.Enabled = False
            Next
            SetHooverIndex(5)
        End If
    End Sub
#End Region

#Region " Private Classes "
    ''' <summary>
    ''' ListViewItem to display a Word document.
    ''' </summary>
    Private Class DocumentListViewItem
        Inherits TemplateListViewItem
        Private m_DocumentAddress As Address
        ''' <summary>
        ''' Gets or sets the recipient address used in the document.
        ''' </summary>
        Property DocumentAddress() As Address
            Get
                Return m_DocumentAddress
            End Get
            Set(ByVal value As Address)
                m_DocumentAddress = value
            End Set
        End Property
        ''' <summary>
        ''' Constructor with path and address of sender and address of recipient.
        ''' </summary>
        ''' <param name="Path">Path of the template.</param>
        ''' <param name="TemplateAddress">Address used in the template (sender address).</param>
        ''' <param name="DocumentAddress">Recipient address.</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal Path As String, ByVal TemplateAddress As Address, ByVal DocumentAddress As Address)
            MyBase.New(Path, TemplateAddress)
            m_DocumentAddress = DocumentAddress
            Me.ImageIndex = 0
        End Sub
    End Class
    ''' <summary>
    ''' ListViewItem to display a Word template.
    ''' </summary>
    Private Class TemplateListViewItem
        Inherits ListViewItem
        Private m_Path As String
        Private m_TemplateAddress As Address
        ''' <summary>
        ''' Returns the address used in the template.
        ''' </summary>
        ReadOnly Property TemplateAddress() As Address
            Get
                Return m_TemplateAddress
            End Get
        End Property
        ''' <summary>
        ''' Returns the path of the template.
        ''' </summary>
        ReadOnly Property Path() As String
            Get
                Return m_Path
            End Get
        End Property
        ''' <summary>
        ''' Constructor with path and address.
        ''' </summary>
        ''' <param name="Path">Path of the template.</param>
        ''' <param name="TemplateAddress">Address used in the template.</param>
        Public Sub New(ByVal Path As String, ByVal TemplateAddress As Address)
            MyBase.New(IO.Path.GetFileNameWithoutExtension(Path))
            m_Path = Path
            m_TemplateAddress = TemplateAddress
            Me.ImageIndex = 0
        End Sub
    End Class
#End Region

End Class
