Namespace Document
    ''' <summary>
    ''' Describes a table cell in the document range.
    ''' </summary>
    Public Class Cell

#Region " Fields "
        ''' <summary>
        ''' Borders listing of the cell.
        ''' </summary>
        Private m_Borders As TableCellBorders
        ''' <summary>
        ''' Vertical alignment of content in this cell.
        ''' </summary>
        Private m_VerticalAlignment As WdCellVerticalAlignment = WdCellVerticalAlignment.NotSet
        ''' <summary>
        ''' Index of assigned table row.
        ''' </summary>
        Private m_RowIndex As Integer
        ''' <summary>
        ''' Index of assigned table column.
        ''' </summary>
        Private m_ColumnIndex As Integer
        ''' <summary>
        ''' Reference of a cell this cell should be merged to.
        ''' </summary>
        Private m_MergeTo As Cell
        ''' <summary>
        ''' Range description for contents of this cell.
        ''' </summary>
        Private m_Range As New Range
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Constructor for a new table cell.
        ''' </summary>
        ''' <param name="RowIndex">Index of assigned table row.</param>
        ''' <param name="ColumnIndex">Index of assigned table column.</param>
        Friend Sub New(ByVal RowIndex As Integer, ByVal ColumnIndex As Integer)
            m_Borders = New TableCellBorders
            m_RowIndex = RowIndex
            m_ColumnIndex = ColumnIndex
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Index of assigned table row.
        ''' </summary>
        Friend ReadOnly Property RowIndex() As Integer
            Get
                Return m_RowIndex
            End Get
        End Property
        ''' <summary>
        ''' Index of assigned table column.
        ''' </summary>
        Friend ReadOnly Property ColumnIndex() As Integer
            Get
                Return m_ColumnIndex
            End Get
        End Property
        ''' <summary>
        ''' Reference of a cell this cell should be merged to.
        ''' </summary>
        Property MergeTo() As Cell
            Get
                Return m_MergeTo
            End Get
            Set(ByVal value As Cell)
                m_MergeTo = value
            End Set
        End Property
        ''' <summary>
        ''' Borders listing of the cell.
        ''' </summary>
        ReadOnly Property Borders() As TableCellBorders
            Get
                Return m_Borders
            End Get
        End Property
        ''' <summary>
        ''' Vertical alignment of content in this cell.
        ''' </summary>
        Property VerticalAlignment() As WdCellVerticalAlignment
            Get
                Return m_VerticalAlignment
            End Get
            Set(ByVal Value As WdCellVerticalAlignment)
                m_VerticalAlignment = Value
            End Set
        End Property
        ''' <summary>
        ''' Range description for contents of this cell.
        ''' </summary>
        ReadOnly Property Range() As Range
            Get
                Return m_Range
            End Get
        End Property
#End Region

    End Class
End Namespace