Namespace Document
    ''' <summary>
    ''' Describes a table column in the document range.
    ''' </summary>
    Public Class Column

#Region " Fields "
        ''' <summary>
        ''' List of cells in this column.
        ''' </summary>
        Private m_Cells As New List(Of Cell)
        ''' <summary>
        ''' Preferred width of this column.
        ''' </summary>
        Private m_PreferredWidth As Single = 0.0F
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Constructor for a new table column.
        ''' </summary>
        Friend Sub New()

        End Sub
        ''' <summary>
        ''' Constructor for a new table column.
        ''' </summary>
        ''' <param name="Tbl">Referenced table.</param>
        ''' <param name="ColumnIndex">Column index.</param>
        Friend Sub New(ByVal Tbl As Table, ByVal ColumnIndex As Integer)
            For I As Integer = 0 To Tbl.RowCount - 1
                m_Cells.Add(Tbl.Cell(I, ColumnIndex))
            Next
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Preferred width of this column.
        ''' </summary>
        Property PreferredWidth() As Single
            Get
                Return m_PreferredWidth
            End Get
            Set(ByVal value As Single)
                m_PreferredWidth = value
            End Set
        End Property
        ''' <summary>
        ''' List of cells in this column.
        ''' </summary>
        ReadOnly Property Cells() As List(Of Cell)
            Get
                Return m_Cells
            End Get
        End Property
        ''' <summary>
        ''' Gets or sets the style of left border.
        ''' </summary>
        WriteOnly Property BorderLeft() As WdLineStyle
            Set(ByVal value As WdLineStyle)
                For Each Cl As Cell In m_Cells
                    Cl.Borders.LeftBorder = value
                Next
            End Set
        End Property
        ''' <summary>
        ''' Gets or sets the style of right border.
        ''' </summary>
        WriteOnly Property BorderRight() As WdLineStyle
            Set(ByVal value As WdLineStyle)
                For Each Cl As Cell In m_Cells
                    Cl.Borders.RightBorder = value
                Next
            End Set
        End Property

#End Region

    End Class

End Namespace
