Namespace Document
    ''' <summary>
    ''' Describes a paragraph in the document range.
    ''' </summary>
    Public Class Paragraph
        Inherits WritableObject

#Region " Fields "
        ''' <summary>
        ''' Format of this paragraph.
        ''' </summary>
        Private m_Format As ParagraphFormat
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Creates a new empty paragraph.
        ''' </summary>
        Friend Sub New()

        End Sub
        ''' <summary>
        ''' Creates a new paragraph with alignment value.
        ''' </summary>
        ''' <param name="Alignment">Alignment value.</param>
        Friend Sub New(ByVal Alignment As WdParagraphAlignment)
            m_Format = New ParagraphFormat
            m_Format.Alignment = Alignment
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Format of this paragraph.
        ''' </summary>
        ReadOnly Property Format() As ParagraphFormat
            Get
                If m_Format Is Nothing Then
                    m_Format = New ParagraphFormat
                End If
                Return m_Format
            End Get
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Writes the content of this paragraph into a Word range.
        ''' </summary>
        ''' <param name="Rng">Active Word range.</param>
        Friend Overrides Sub Write(ByRef Rng As Object)
            Rng.Paragraphs.Add()
            Rng.Collapse(WdCollapseDirection.wdCollapseEnd)

            If m_Format IsNot Nothing Then
                m_Format.Write(Rng)
            End If

        End Sub
        ''' <summary>
        ''' Releases marshalled objects.
        ''' </summary>
        Private Sub ReleaseObject(ByVal obj As Object)
            If obj IsNot Nothing Then
                Try
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)

                Catch
                Finally
                    obj = Nothing
                End Try
            End If
        End Sub
#End Region

    End Class

End Namespace