Namespace Document
    ''' <summary>
    ''' Formatting properties for a paragraph or range.
    ''' </summary>
    Public Class ParagraphFormat

#Region " Fields "
        ''' <summary>
        ''' Alignment of this paragraph.
        ''' </summary>
        Private m_Alignment As WdParagraphAlignment = WdParagraphAlignment.NotSet
        ''' <summary>
        ''' Line space before this paragraph.
        ''' </summary>
        Private m_SpaceBefore As Single = -1.0F
        ''' <summary>
        ''' Line space after this paragraph.
        ''' </summary>
        Private m_SpaceAfter As Single = -1.0F
        ''' <summary>
        ''' Line spacing rule of this paragraph.
        ''' </summary>
        Private m_LineSpacingRule As WdLineSpacing = WdLineSpacing.NotSet
#End Region

#Region " Properties "
        ''' <summary>
        ''' Sets the line spacing rule of this paragraph.
        ''' </summary>
        Property LineSpacingRule() As WdLineSpacing
            Get
                Return m_LineSpacingRule
            End Get
            Set(ByVal value As WdLineSpacing)
                m_LineSpacingRule = value
            End Set
        End Property
        ''' <summary>
        ''' Sets the line space before this paragraph.
        ''' </summary>
        Property SpaceBefore() As Single
            Get
                Return m_SpaceBefore
            End Get
            Set(ByVal value As Single)
                m_SpaceBefore = value
            End Set
        End Property
        ''' <summary>
        ''' Sets the line space after this paragraph.
        ''' </summary>
        Property SpaceAfter() As Single
            Get
                Return m_SpaceAfter
            End Get
            Set(ByVal value As Single)
                m_SpaceAfter = value
            End Set
        End Property
        ''' <summary>
        ''' Sets the alignment of this paragraph.
        ''' </summary>
        Property Alignment() As WdParagraphAlignment
            Get
                Return m_Alignment
            End Get
            Set(ByVal value As WdParagraphAlignment)
                m_Alignment = value
            End Set
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Writes the content of this format into a Word range.
        ''' </summary>
        Public Sub Write(ByVal Rng As Object)
            Dim ParagraphFormat As Object = Rng.ParagraphFormat

            If m_Alignment <> WdParagraphAlignment.NotSet Then
                ParagraphFormat.Alignment = m_Alignment
            End If

            If m_SpaceBefore <> -1.0F Then
                ParagraphFormat.SpaceBefore = CSng(m_SpaceBefore)
            End If
            If m_SpaceAfter <> -1.0F Then
                ParagraphFormat.SpaceAfter = CSng(m_SpaceAfter)
            End If
            If m_LineSpacingRule <> WdLineSpacing.NotSet Then
                ParagraphFormat.LineSpacingRule = m_LineSpacingRule
            End If

            ReleaseObject(ParagraphFormat)
        End Sub
        ''' <summary>
        ''' Releases marshalled objects.
        ''' </summary>
        Private Sub ReleaseObject(ByVal obj As Object)
            If obj IsNot Nothing Then
                Try
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)

                Catch
                Finally
                    obj = Nothing
                End Try
            End If
        End Sub
#End Region

    End Class
End Namespace