Namespace Document
    ''' <summary>
    ''' Describes a separately formatted string in the document range.
    ''' </summary>
    Public Class TextSegment
        Inherits WritableObject

#Region " Fields "
        ''' <summary>
        ''' Text of this segment.
        ''' </summary>
        Private m_Text As String
        ''' <summary>
        ''' Font of this segment.
        ''' </summary>
        Private m_Font As Font = Nothing
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Constructor
        ''' </summary>
        Friend Sub New()
            m_Text = ""
        End Sub
        ''' <summary>
        ''' Constructor
        ''' </summary>
        Friend Sub New(ByVal Text As String)
            m_Text = Text
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Font of this segment.
        ''' </summary>
        ReadOnly Property Font() As Font
            Get
                If m_Font Is Nothing Then
                    m_Font = New Font
                End If
                Return m_Font
            End Get
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Writes the content of this segment into a Word range.
        ''' </summary>
        ''' <param name="Rng">Active Word range.</param>
        Friend Overrides Sub Write(ByRef Rng As Object)
            Rng.Text = m_Text

            If m_Font IsNot Nothing Then
                Dim Ft As Object = Rng.Font
                Ft.Bold = m_Font.Bold
                Ft.Italic = m_Font.Italic
                Ft.Underline = m_Font.Underline
                If m_Font.Size <> 0.0F Then
                    Ft.Size = m_Font.Size
                End If
                If m_Font.Name <> "" Then
                    Ft.Name = m_Font.Name
                End If
                ReleaseObject(Ft)
            End If

            Rng.Collapse(WdCollapseDirection.wdCollapseEnd)
        End Sub
        ''' <summary>
        ''' Releases marshalled objects.
        ''' </summary>
        Private Sub ReleaseObject(ByVal obj As Object)
            If obj IsNot Nothing Then
                Try
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)

                Catch
                Finally
                    obj = Nothing
                End Try
            End If
        End Sub
#End Region

    End Class
End Namespace