Imports System.ComponentModel
Namespace Document

    ''' <summary>
    ''' Collection of bookmarks in a range description.
    ''' </summary>
    <EditorBrowsable(EditorBrowsableState.Advanced)> _
    Public Class Bookmarks
        Inherits RangeItemsList(Of Bookmark)

        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="Rng">Referenced range description.</param>
        Friend Sub New(ByVal Rng As Range)
            MyBase.New(Rng)
        End Sub
        ''' <summary>
        ''' Adds a bookmark to this range description and returns it.
        ''' </summary>
        ''' <param name="Name">Bookmark name.</param>
        Public Function Add(ByVal Name As String) As Bookmark
            Dim Result As Bookmark = FindBookmark(Name, True)
            If Result Is Nothing Then
                Result = New Bookmark(Name)
                MyBase.AddObject(Result)
            End If
            Return Result
        End Function
        ''' <summary>
        ''' Finds a bookmark in  hierarchical range descriptions.
        ''' </summary>
        Private Function FindBookmark(ByVal Name As String, ByVal IncludeChildren As Boolean) As Bookmark
            For Each Obj As Bookmark In Me
                If Obj.Name = Name Then
                    Return Obj
                End If
            Next
            If IncludeChildren Then
                Dim Result As Bookmark = Nothing
                For Each Tbl As Table In Me.Range.Tables
                    For Each cl As Cell In Tbl.Cells
                        Result = cl.Range.Bookmarks.FindBookmark(Name, True)
                        If Result IsNot Nothing Then
                            Return Result
                        End If
                    Next
                Next
            End If

            Return Nothing
        End Function
        ''' <summary>
        ''' Returns a bookmark by name.
        ''' </summary>
        Default Overloads ReadOnly Property Item(ByVal Name As String) As Bookmark
            Get
                Dim Found As Bookmark = FindBookmark(Name, True)
                If Found IsNot Nothing Then
                    Return Found
                End If

                Dim Result As New Bookmark(Name)
                MyBase.AddObject(Result)
                Return Result
            End Get
        End Property
    End Class
End Namespace