Namespace Document
    ''' <summary>
    ''' Collection of objects to write into a Word range.
    ''' </summary>
    Public Class Range

#Region " Fields "
        ''' <summary>
        ''' List of all writable contents in this range.
        ''' </summary>
        Private m_Contents As New List(Of WritableObject)
        ''' <summary>
        ''' List of tables in this range.
        ''' </summary>
        Private m_Tables As Tables
        ''' <summary>
        ''' List of paragraphs in this range.
        ''' </summary>
        Private m_Paragraphs As Paragraphs
        ''' <summary>
        ''' List of bookmarks in this range.
        ''' </summary>
        Private m_Bookmarks As Bookmarks
        ''' <summary>
        ''' List of breaks in this range.
        ''' </summary>
        Private m_Breaks As Breaks
        ''' <summary>
        ''' List of text segments in this range.
        ''' </summary>
        Private m_Text As Text
        ''' <summary>
        ''' First paragraph format in this range.
        ''' </summary>
        Private WithEvents m_Format As ParagraphFormat
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Constructor
        ''' </summary>
        Public Sub New()
            m_Tables = New Tables(Me)
            m_Paragraphs = New Paragraphs(Me)
            m_Bookmarks = New Bookmarks(Me)
            m_Breaks = New Breaks(Me)
            m_Text = New Text(Me)
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Sets the first paragraph format in this range.
        ''' </summary>
        ReadOnly Property Format() As ParagraphFormat
            Get
                If m_Format Is Nothing Then
                    m_Format = New ParagraphFormat
                End If
                Return m_Format
            End Get
        End Property
        ''' <summary>
        ''' List of writable contents in this range.
        ''' </summary>
        Friend ReadOnly Property Contents() As List(Of WritableObject)
            Get
                Return m_Contents
            End Get
        End Property
        ''' <summary>
        ''' List of tables in this range.
        ''' </summary>
        ReadOnly Property Tables() As Tables
            Get
                Return m_Tables
            End Get
        End Property
        ''' <summary>
        ''' List of paragraphs in this range.
        ''' </summary>
        ReadOnly Property Paragraphs() As Paragraphs
            Get
                Return m_Paragraphs
            End Get
        End Property
        ''' <summary>
        ''' List of bookmarks in this range.
        ''' </summary>
        Overridable ReadOnly Property Bookmarks() As Bookmarks
            Get
                Return m_Bookmarks
            End Get
        End Property
        ''' <summary>
        ''' List of breaks in this range.
        ''' </summary>
        ReadOnly Property Breaks() As Breaks
            Get
                Return m_Breaks
            End Get
        End Property
        ''' <summary>
        ''' List of text segments in this range.
        ''' </summary>
        ReadOnly Property Text() As Text
            Get
                Return m_Text
            End Get
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Converts centimeter to point values.
        ''' </summary>
        Shared Function CentimetersToPoints(ByVal Value As Single) As Single
            Return Value * 28.35F
        End Function
        ''' <summary>
        ''' Writes the contents of this range description into a Word document range.
        ''' </summary>
        ''' <param name="Rng">Word range.</param>
        Friend Sub Write(ByRef Rng As Object)
            If m_Format IsNot Nothing Then
                m_Format.Write(Rng)
            End If
            If m_Contents.Count = 0 Then
                Rng.Text = ""
            Else
                For Each Obj As WritableObject In m_Contents
                    Obj.Write(Rng)
                Next
            End If

        End Sub
#End Region

    End Class

End Namespace

